package main

import (
	"database/sql"
	"log"

	"bitbucket.org/prmunty/util/dbmigration/db"
	"bitbucket.org/prmunty/util/dbmigration/sqliteutils"
	_ "github.com/mattn/go-sqlite3"
)

const version float64 = 3.0

type OldVal struct {
	Selected bool
	FreezeID int
}

type NewVal struct {
	FreezeID int
	Section  int
}

var newVals = []NewVal{
	{FreezeID: 2085, Section: 1},
	{FreezeID: 2090, Section: 1},
	{FreezeID: 2091, Section: 1},
	{FreezeID: 2092, Section: 1},
	{FreezeID: 2093, Section: 1},
	{FreezeID: 2094, Section: 1},
	{FreezeID: 2095, Section: 1},
	{FreezeID: 2096, Section: 1},
	{FreezeID: 2097, Section: 1},
	{FreezeID: 2249, Section: 1},
	{FreezeID: 2257, Section: 1},

	{FreezeID: 2085, Section: 2},
	{FreezeID: 2090, Section: 2},
	{FreezeID: 2091, Section: 2},
	{FreezeID: 2092, Section: 2},
	{FreezeID: 2093, Section: 2},
	{FreezeID: 2094, Section: 2},
	{FreezeID: 2095, Section: 2},
	{FreezeID: 2096, Section: 2},
	{FreezeID: 2097, Section: 2},
	{FreezeID: 2249, Section: 2},
	{FreezeID: 2257, Section: 2},

	{FreezeID: 2085, Section: 3},
	{FreezeID: 2090, Section: 3},
	{FreezeID: 2091, Section: 3},
	{FreezeID: 2092, Section: 3},
	{FreezeID: 2093, Section: 3},
	{FreezeID: 2094, Section: 3},
	{FreezeID: 2095, Section: 3},
	{FreezeID: 2096, Section: 3},
	{FreezeID: 2097, Section: 3},
	{FreezeID: 2249, Section: 3},
	{FreezeID: 2257, Section: 3},

	{FreezeID: 134, Section: 4},
	{FreezeID: 135, Section: 4},
	{FreezeID: 136, Section: 4},
	{FreezeID: 137, Section: 4},
	{FreezeID: 138, Section: 4},
	{FreezeID: 139, Section: 4},
	{FreezeID: 140, Section: 4},
	{FreezeID: 141, Section: 4},
	{FreezeID: 142, Section: 4},
	{FreezeID: 143, Section: 4},
	{FreezeID: 2343, Section: 4},
	{FreezeID: 2344, Section: 4},
	{FreezeID: 2345, Section: 4},
	{FreezeID: 2534, Section: 4},
	{FreezeID: 2535, Section: 4},
	{FreezeID: 2536, Section: 4},
	{FreezeID: 2537, Section: 4},
}

func main() {
	db.Migrate(version, func(tx *sql.Tx) error {
		sqliteDB := sqliteutils.DB{tx}
		if sqliteDB.ColumnExists("freeze_data_vessel", "section") {
			return nil
		}

		oldVals, err := getOldVals(tx)
		if err != nil {
			return err
		}

		if err = modifyTable(tx); err != nil {
			return nil
		}

		return updateValues(tx, oldVals, newVals)

	})
}

func getOldVals(tx *sql.Tx) (vals map[int]OldVal, err error) {
	vals = make(map[int]OldVal)
	rows, err := tx.Query("SELECT freeze_id, selected from freeze_data_vessel")
	if err != nil {
		return vals, err
	}

	for rows.Next() {
		var currVal OldVal
		if err = rows.Scan(&currVal.FreezeID, &currVal.Selected); err != nil {
			log.Println(err)
			continue
		}

		vals[currVal.FreezeID] = currVal
	}

	return vals, err
}

func updateValues(tx *sql.Tx, oldVals map[int]OldVal, newVals []NewVal) error {
	stmt, err := tx.Prepare("INSERT INTO freeze_data_vessel VALUES(?,?,?)")
	if err != nil {
		return err
	}

	for _, newVal := range newVals {
		if _, err = stmt.Exec(newVal.FreezeID, newVal.Section, oldVals[newVal.FreezeID].Selected); err != nil {
			return err
		}
	}

	return nil
}

func modifyTable(tx *sql.Tx) error {
	_, err := tx.Exec(`DROP TABLE IF EXISTS freeze_data_vessel;
             CREATE TABLE freeze_data_vessel(
                 freeze_id INTEGER,  
                 section INTEGER,
                 selected BOOLEAN,
                 PRIMARY KEY (freeze_id, section)
             );
           `)

	return err
}
